#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Metric < ActiveRecord::Base
  
  DOMAIN_RULES = 'Rules'
  
  VALUE_TYPE_INT = 'INT'
  VALUE_TYPE_BOOLEAN = 'BOOL'
  VALUE_TYPE_FLOAT = 'FLOAT'
  VALUE_TYPE_PERCENT = 'PERCENT'
  VALUE_TYPE_STRING = 'STRING'
  VALUE_TYPE_DATA = 'DATA'
  VALUE_TYPE_MILLISEC = 'MILLISEC'
  VALUE_TYPE_LEVEL = 'LEVEL'
  
  TYPE_LEVEL_OK = 'OK'
  TYPE_LEVEL_WARN = 'WARN'
  TYPE_LEVEL_ERROR = 'ERROR'

  ORIGIN_GUI='GUI'
  ORIGIN_JAVA='JAV'

  validates_length_of       :name, :within => 1..64
  validates_uniqueness_of   :name
  validates_length_of       :short_name, :within => 1..64
  validates_inclusion_of    :val_type, :in => [VALUE_TYPE_INT,VALUE_TYPE_BOOLEAN,VALUE_TYPE_FLOAT,VALUE_TYPE_PERCENT,VALUE_TYPE_STRING,VALUE_TYPE_MILLISEC,VALUE_TYPE_LEVEL], :message => "wrong value type"


  @@metrics_type_names = { VALUE_TYPE_INT => 'Integer', VALUE_TYPE_FLOAT => 'Float', VALUE_TYPE_PERCENT => 'Percent',
    VALUE_TYPE_BOOLEAN => 'Yes/No', VALUE_TYPE_STRING => 'Text', VALUE_TYPE_LEVEL => 'Level' }

  attr_accessible :name, :description, :direction, :domain, :short_name, :qualitative, :val_type, :user_managed

  def ==(other)
    name==other.name
  end

  def <=>(other)
    short_name<=>other.short_name
  end

  def self.domains
    all.collect{|metric|
      metric.domain
    }.compact.uniq.sort
  end
  
  def key
    name
  end

  def user_managed?
    user_managed==true
  end

  def value_type
    val_type
  end

  def numeric?
    val_type==VALUE_TYPE_INT or val_type==VALUE_TYPE_FLOAT or val_type==VALUE_TYPE_PERCENT or val_type==VALUE_TYPE_MILLISEC
  end

  def qualitative?
    qualitative
  end

  def value_type_name
    @@metrics_type_names[value_type]
  end
  
  def self.value_type_names
    # return a copy of the hash
    metrics_type_names_clone = {}
    @@metrics_type_names.each_pair {|type, descr| 
      metrics_type_names_clone[type] = descr
    }
    metrics_type_names_clone  
  end

  def timemachine?
    val_type != VALUE_TYPE_DATA
  end

  def display?
    enabled && val_type!=VALUE_TYPE_DATA && key!=ALERT_STATUS
  end

  def alertable?
    (val_type != VALUE_TYPE_DATA) and (key!=Metric::ALERT_STATUS)
  end

  def treemap?
    val_type==VALUE_TYPE_PERCENT && qualitative? && enabled
  end

  def suffix
    case val_type
      when VALUE_TYPE_PERCENT
        '%'
      when VALUE_TYPE_MILLISEC
        'ms'
      else ''
    end
  end

  def self.all
    cache.get('ALL')
  end
   
  def self.by_id(id)
    cache.get(id.to_s)
  end
  
  def self.by_name(key)
    cache.get(key)
  end

  def self.by_key(key)
    cache.get(key.to_s)
  end
  
  def self.by_names(names)
    metrics = names.map do |name|
      by_name(name)
    end
    metrics
  end
  
  def self.by_domain(domain)
    all.select{|metric| metric.domain==domain}.sort
  end
  
  def self.major_metrics_id
    all.collect {|m| m.id}
  end
  
  def self.clear_cache
    @@c = nil
    Caches.clear(Caches::METRICS)
  end
  
  def self.default_time_machine_metrics
    [CYCLOMATIC_COMPLEXITY, CODE_COVERAGE, RULES_MANDATORY_INDEX]
  end

  def self.review_types
    all.select{|metric| metric.user_managed?}
  end

  def self.ids_from_keys(keys_array)
    keys_array.collect{ |key| Metric.by_name(key).id if Metric.by_name(key) }
  end

  def to_hash_json(options={})
    return {'key' => name, 'name' => short_name, 'description' => description, 'domain' => domain,
      'qualitative' => qualitative.to_s, 'user_managed' => self.user_managed.to_s,
      'direction' => direction.to_s, 'val_type' => val_type.to_s}
  end

  def to_xml(options={})
    xml = Builder::XmlMarkup.new
    xml.metric do
      xml.key(name)
      xml.name(short_name)
      xml.description(description)
      xml.domain(domain)
      xml.qualitative(qualitative)
      xml.direction(direction)
      xml.user_managed(self.user_managed)
      xml.val_type(val_type)
    end
  end

  def created_online?
    origin==ORIGIN_GUI
  end

  def updatable_online?
    origin!=ORIGIN_JAVA
  end


  # METRIC DEFINITIONS
  # WARNING if you edit this file do not forget to also change sonar-commons/src/main/java/ch/hortis/sonar/model/Metrics.java
  
  COBERTURA_LINE_COVERAGE = 'cobertura_line_rate'
  COBERTURA_BRANCH_COVERAGE = 'cobertura_branch_rate'
  CLOVER_COVERAGE = 'clover_code_coverage'
  
  NCSS_PACKAGES = 'packages_count'
  NCSS_CLASSES = 'classes_count'
  NCSS_NCSS = 'ncss'
  NCSS_FUNCTIONS = 'functions_count'
  FILES_COUNT='files_count'
  DIRECTORIES_COUNT='directories_count'
  
  CYCLOMATIC_COMPLEXITY = 'ccn'
  CC_AVG_CLASS = 'ccn_class'
  CC_AVG_FUNCTION = 'ccn_function'
  CC_AVG_FILE = 'ccn_file'
  CC_CLASSES_COUNT_DISTRIB = 'ccn_classes_count_distribution'
  CC_CLASSES_PERCENT_DISTRIB = 'ccn_classes_percent_distribution'
  
  PMD_DUPLICATION = 'duplication'
  PMD_DUPLICATED_LINES = 'duplicated_lines'
  PMD_DUPLICATED_TOKENS = 'duplicated_tokens'
  DUPLICATED_LINES_RATIO = 'duplicated_lines_ratio'
  
  SUREFIRE_ERRORS = 'test_errors_count'
  SUREFIRE_SKIPPED = 'test_skipped_count'
  SUREFIRE_FAILURES = 'test_failures_count'
  SUREFIRE_TESTS = 'test_count'
  SUREFIRE_TIME = 'test_execution_time'
  SUREFIRE_SUCCESS_PERCENTAGE = 'test_success_percentage'
  SUREFIRE_ERRORS_PERCENTAGE = 'test_errors_percentage'
  SUREFIRE_FAILURE_PERCENTAGE = 'test_failure_percentage'
  
  CODE_COVERAGE = 'code_coverage'
  RULES_INDEX = 'rules_index'
  RULES_VIOLATIONS_COUNT = 'rules_violations_count'
  RULES_MANDATORY_INDEX = 'rules_compliance'
  RULES_MANDATORY_VIOLATIONS_COUNT = 'rules_violations'
  RULES_OPTIONAL_INDEX = 'optional_rules_compliance'
  RULES_OPTIONAL_VIOLATIONS_COUNT = 'optional_rules_violations'

  FAILURE_RULES_INDEX = 'strict_rules_compliance'
  
  COMMENT_RATIO = 'comment_ratio'
  COMMENT_LINES = 'comment_lines'
  
  ALERT_STATUS = 'alert_status'
  PROFILE='profile'
  
  private


  def self.cache
    @@c ||= Caches.cache(Caches::METRICS)
    if @@c.size==0
      metrics=Metric.find(:all, :conditions => {:enabled => true})
      metrics.each do |metric|
        @@c.put(metric.name, metric)
        @@c.put(metric.id.to_s, metric)
      end
      @@c.put('ALL', metrics)
    end
    @@c
  end

  def self.coding_rules_to_ignore
    %w{ 'rules_violations', 'optional_rules_compliance', 'optional_rules_violations' }
  end
  
end