#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Project < ActiveRecord::Base
  include Comparable

  has_many :snapshots
  has_many :processed_snapshots, :class_name => 'Snapshot', :conditions => "status='#{Snapshot::STATUS_PROCESSED}'", :order => 'created_at desc'
  has_many :events, :foreign_key => 'resource_id', :order => 'event_date DESC'
  has_many :project_links, :dependent => :delete_all, :order => 'link_type'
  belongs_to :profile, :class_name => 'Profile', :foreign_key => 'profile_id'

  SCOPE_PROJECT='PRJ'
  SCOPE_PACKAGE='PAC'
  SCOPE_CLASS='CLA'
  SORTED_SCOPES=[SCOPE_PROJECT, SCOPE_PACKAGE, SCOPE_CLASS]

  def self.by_key(k)
    ki=k.to_i
    if ki==0
      Project.find(:first, :conditions => {:enabled => true, :kee => k})
    else
      Project.find(ki)
    end
  end

  def last_snapshot
    @last_snapshot ||=
      begin
      Snapshot.find(:first, :conditions => {:islast => true, :project_id => id.to_i}, :include => 'project')
    end
  end

  def events_with_snapshot
    events.select{|event| not event.snapshot_id.nil?}
  end
  
  def key
    kee
  end
  
  def project?
    scope==SCOPE_PROJECT
  end

  def package?
    scope==SCOPE_PACKAGE
  end
  
  def class?
    scope==SCOPE_CLASS
  end
  
  def last_measure(metric_key, rules_category_id=nil, rule_id=nil)
    last_snapshot ? last_snapshot.measure( metric_key, rules_category_id, rule_id ) : nil
  end
  
  def last_formatted_measure(metric_name, rules_category_id=nil, rule_id=nil)
    measure = last_measure(metric_name, rules_category_id, rule_id)
    measure ? measure.formatted_value : ''
  end
  
  def last_tendency(metric_name, rules_category_id=nil, rule_id=nil)
    last_snapshot ? last_snapshot.tendency( metric_name, rules_category_id, rule_id ) : nil
  end  
  
  def last_measure_value(metric_name, rules_category_id=nil, rule_id=nil)
    measure = last_measure(metric_name, rules_category_id, rule_id)
    measure ? measure.value : measure
  end

  def links
    project_links
  end
  
  def link(type)
    # to_a avoids conflicts with ActiveRecord:Base.find
    links.to_a.find{ |l| l.link_type==type}
  end 
  
  def custom_links
    links.select {|l| l.custom?}
  end
  
  def standard_links
    links.reject {|l| l.custom?}
  end
  
  def chart_measures(metric_id)
    sql = Project.send(:sanitize_sql, ['select s.created_at as created_at, m.value as value ' +
          ' from project_measures m, snapshots s ' +
          ' where s.id=m.snapshot_id and ' +
          " s.status='%s' and " +
          ' s.project_id=%s and m.metric_id=%s ', Snapshot::STATUS_PROCESSED, self.id, metric_id] ) +
      ' and m.rules_category_id IS NULL and m.rule_id IS NULL' +
      ' order by s.created_at'
    create_chart_measures( Project.connection.select_all( sql ), 'created_at', 'value' )
  end
  
  def <=>(other)
    kee <=> other.kee
  end

  def fullname
    branch_name = branch
    if branch_name.nil?
      name
    else
      "#{name} #{branch_name}"
    end
  end
  
  def branch
    if project?
      s=kee.split(':')
      if s.size>=3
        return s[2]
      end 
    end
    nil
  end

  def to_hash_json(options, snapshots=[], measures_by_snapshot_id={})
    json = {'id' => id, 'key' => kee, 'name' => fullname, 'qualifier' => qualifier, 'scope' => scope}
    json['description']=description if description
    json['branch']=branch if branch
    json['links'] = links.collect{|project_link| project_link.to_hash_json} if options[:includelinks] == "true"
    json['measures'] = snapshots.collect{|snapshot| snapshot.to_hash_json(options, measures_by_snapshot_id[snapshot.id])} if snapshots and snapshots.size>0
    json
  end

  def to_xml(options, snapshots=[], measures_by_snapshot_id={})
    xml = Builder::XmlMarkup.new
    xml.project do
      xml.id(id)
      xml.key(kee)
      xml.name(fullname)
      xml.description(description) if description
      xml.qualifier(qualifier)
      xml.branch(branch) if branch
      xml.scope(scope)
      if options[:includelinks] == 'true'
        links.each do |project_link|
          xml << project_link.to_xml
        end
      end
      snapshots.each do |snapshot|
        xml << snapshot.to_xml(options, measures_by_snapshot_id[snapshot.id])
      end
    end
  end

  
  private
  
  def create_chart_measures(results, date_column_name, value_column_name)
    chart_measures = []
    if results and results.first != nil
      # :sanitize_sql is protected so its behaviour cannot be predicted exactly,
      # the jdbc active record impl adapter returns a db typed objects array
      # when regular active record impl return string typed objects
      if results.first[date_column_name].class == Time
        results.each do |hash|
          chart_measures << ChartMeasure.new( hash[date_column_name], hash[value_column_name] )
        end
      else
        results.each do |hash|
          chart_measures << ChartMeasure.new( Time.parse( hash[date_column_name] ), hash[value_column_name].to_d )
        end
      end
    end
    chart_measures    
  end
end
