#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Snapshot < ActiveRecord::Base
  acts_as_tree :foreign_key => 'parent_snapshot_id'

  belongs_to :project
  has_many :measures, :class_name => 'ProjectMeasure', :include => [:rule, :rules_category]
  has_many :events, :dependent => :destroy, :order => 'event_date DESC'
  has_one :source, :class_name => 'SnapshotSource', :dependent => :destroy

  has_many :async_measure_snapshots
  has_many :async_measures, :through => :async_measure_snapshots

  STATUS_UNPROCESSED = 'U'
  STATUS_PROCESSED = 'P'

  def self.last_enabled_projects
    Snapshot.find(:all, :include => 'project',
      :conditions => ['islast=? and projects.enabled=? and parent_snapshot_id is null and snapshots.scope=? and snapshots.status=?',
        true, true, Project::SCOPE_PROJECT, STATUS_PROCESSED])
  end

  def self.for_timemachine_matrix(resource_id)
    snapshots=Snapshot.find(:all, :conditions => ["snapshots.project_id=? AND events.snapshot_id=snapshots.id AND snapshots.status=?", resource_id, STATUS_PROCESSED],
       :include => 'events',
       :order => 'snapshots.created_at ASC')
       
    snapshots=snapshots[-5,5] if snapshots.size>=5
    
    snapshots.insert(0, Snapshot.find(:first, 
         :conditions => ["project_id = :project_id AND projects.enabled = :enabled AND status=:status", {:project_id => resource_id, :enabled => true, :status => STATUS_PROCESSED}],
         :include => 'project', :order => 'snapshots.created_at ASC', :limit => 1))
    snapshots.compact.uniq
  end

  def last?
    islast
  end

  def purged?
    purged
  end

  def root?
    parent_snapshot_id.nil?
  end

  def all_measures
    measures + async_measures
  end

  def descendants
    children.map(&:descendants).flatten + children
  end

  def formatted_label(default='')
    if label
      label.name
    else
      default
    end
  end

  def display_trends?
    islast and (scope==Project::SCOPE_PROJECT or scope==Project::SCOPE_PACKAGE)
  end

  def qualifier_label
    case qualifier
    when 'PLS'
      return 'PL/SQL'
    when 'JAV'
      return 'Java'
    end
    return ''
  end
  
  def event(category)
    result=events.select{|e| e.category==category}
    if result.empty?
      nil
    else 
      result.first
    end
  end

  def metrics
    if @metrics.nil?
      @metrics = []
      all_measures.each do |measure|
        @metrics << measure.metric
      end
      @metrics.uniq!
    end
    @metrics
  end

  def measure(metric_key, rules_category_id=nil, rule_id=nil)
    metric=Metric.by_name(metric_key)
    if metric
      key=[metric.id, rules_category_id, rule_id].join('|')
      return measures_hash[key]
    end
    nil
  end

  def f_measure(metric_key, rules_category_id=nil, rule_id=nil, empty_label='')
    m=measure(metric_key, rules_category_id, rule_id)
    (m.nil?) ? empty_label : m.formatted_value
  end

  def formatted_measure(metric_name, rules_category_id=nil, rule_id=nil)
    m=measure(metric_name, rules_category_id, rule_id)
    (m.nil?) ? '' : m.formatted_value
  end

  def rules_category_measures(metric_key)
    measures.select do |measure|
      measure.rule.nil? and measure.rules_category and measure.metric and measure.metric.key==metric_key
    end
  end

  def rule_measures(metric, category_id=nil)
    result = []
    for measure in all_measures do
      if (measure.rule_id and measure.metric == metric)
        if category_id
          result << measure if measure.rules_category_id==category_id
        else
          result << measure
        end
      end
    end

    ProjectMeasure.sort_by_metric_direction(result, metric)
  end

  def created_at_in_secs
    created_at.tv_sec
  end

  def formatted_date
    created_at.strftime('%b. %e, %Y %H:%M')
  end

  def formatted_day
    created_at.strftime('%b. %e, %Y')
  end

  def rest_date
    created_at.strftime('%Y-%m-%dT%H:%M:%S')
  end

  def label_name
    !label.nil? ? label.name : formatted_date
  end

  def label_for_html_id
    !label.nil? ? label.name : created_at.strftime('%Y%m%d_%H%M%S')
  end

  def to_hash_json(options, measures=[])
    json = {}
    json['sid'] = id
    json['date'] = rest_date
    json['label'] = label.name if options['includelabels']=='true' and label
    json['version'] = version.to_s if version
    json['msr'] = measures.collect {|measure| measure.to_hash_json(options)} if measures
    json
  end

  def to_xml(options, measures=[])
    xml = Builder::XmlMarkup.new
    xml.measures do
      xml.sid(id)
      xml.date(rest_date)
      xml.label(label.name) if options['includelabels']=='true' and label
      xml.version(version) if version
      if measures
        measures.each do |measure|
          xml << measure.to_xml(options)
        end
      end
    end
  end


  def self.snapshot_by_date(resource_id, date)
    if resource_id and date
      Snapshot.find(:first, :conditions => ['created_at>=? and created_at<? and project_id=?', date.beginning_of_day, date.end_of_day, resource_id], :order => 'created_at desc')
    else
      nil
    end
  end

  private

  def measures_hash
    @measures_hash ||=
      begin
        hash = {}
        all_measures.each do |measure|
          hash[measure_key(measure)]=measure
        end
        hash
      end
  end

  def measure_key(pm)
    [pm.metric_id, pm.rules_category_id, pm.rule_id].join('|')
  end


end