 #
 # Sonar, entreprise quality control tool.
 # Copyright (C) 2007-2008 Hortis-GRC SA
 # mailto:be_agile HAT hortis DOT ch
 #
 # Sonar is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 3 of the License, or (at your option) any later version.
 #
 # Sonar is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with Sonar; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 #
class Sonar::Treemap 
  include ActionView::Helpers::JavaScriptHelper
  
  attr_accessor :color_name, :size_name, :width, :height, :link_url, :display_branch_name
  attr_accessor :measures_by_snapshot
  
  DEFAULT_WIDTH = 280
  DEFAULT_HEIGHT = 280

  CONFIGURATION_DEFAULT_COLOR_METRIC = 'sonar.core.treemap.colormetric'
  CONFIGURATION_DEFAULT_SIZE_METRIC = 'sonar.core.treemap.sizemetric'
  
  def initialize(measures_by_snapshot, options={})
    options ||= {}
    options[:display_branch_name] = options[:display_branch_name] ? options[:display_branch_name] : false
    
    @measures_by_snapshot = get_measures_by_snapshot_without_rule_and_rules_category(measures_by_snapshot)
    @size_name = options[:size_name] ||= default_size_metric
    @color_name = options[:color_name] ||= default_color_metric
    @display_branch_name = display_branch_name
    @width = options[:width] ? options[:width].to_i : DEFAULT_WIDTH
    @height = options[:height] ? options[:height].to_i : DEFAULT_HEIGHT
    @link_url = options[:link_url]

    color_metric=Metric.by_name(@color_name)
    @direction = (color_metric ? color_metric.direction : 1)
    
    # restriction : color metric values from 0 to 100
    @min = 0
    @max = 100
    @min_color = Color::RGB.from_html("FF0000")   # red
    @mean_color = Color::RGB.from_html("FFB000")   # orange
    @max_color = Color::RGB.from_html("00FF00")   # green
  end

  def self.default_color_metric
    Metric::RULES_MANDATORY_INDEX
  end

  def self.default_size_metric
    Metric::NCSS_NCSS
  end
  
  def enough_data?
    @measures_by_snapshot and @measures_by_snapshot.size > 0
  end
  
  def generate_html
    root = Treemap::Node.new( :id => -1, :label => "" )
    build_tree( @measures_by_snapshot, root, @size_name, @color_name )
    
    output = Sonar::HtmlOutput.new do |o|
      o.width = @width
      o.height = @height
      o.full_html = false
      o.link_url = @link_url
      o.details_at_depth = 1
    end
    output.to_html( root )
  end
  
  def self.size_metrics
    [Metric.by_name(Metric::NCSS_NCSS), Metric.by_name(Metric::CYCLOMATIC_COMPLEXITY), Metric.by_name(Metric::CC_AVG_FUNCTION)].sort {|m1, m2| m1.short_name <=> m2.short_name}
  end

  def self.color_metrics
    Metric.all.select { |metric| metric.treemap?}
  end

  def self.get_color_metrics_per_domain
    get_metrics_per_domain(color_metrics)
  end

  def self.get_color_metric_direction(options)
    color_name = options ? options[:color_name] : default_color_metric
    color_metric=Metric.by_name(color_name)
    color_metric ? color_metric.direction : 1
  end

  
  protected

  def get_measures_by_snapshot_without_rule_and_rules_category(measures_by_snapshot)
    new_hash = {}
    if measures_by_snapshot
      measures_by_snapshot.each_pair do |snapshot, measures|
        new_hash[snapshot] = measures.reject{|measure| !measure.rule_id.nil? or !measure.rules_category_id.nil?}
      end
    end
    new_hash
  end

  def build_tree( measures_by_snapshot, node, size_metric_key, color_metric_key )    
    measures_by_snapshot.each_pair do |snapshot, measures|
      size_measure = get_measure(size_metric_key, measures)
      color_measure = get_measure(color_metric_key, measures)            
      if size_measure and color_measure
        project = snapshot.project
        metric_value = color_measure.value
        project_label = (display_branch_name ? project.fullname : project.name)
     
        child = Treemap::Node.new( :id => project.id, 
          :size => size_measure.value.to_i, 
          :label => project_label, 
          :tooltip => get_html_tooltip(project, measures_by_snapshot[snapshot]),
          :color => get_hex_color( metric_value),
          :url => get_url(snapshot))
        node.add_child(child)
      end
    end
  end 
    
  def get_measure(metric_key, measures)
    metric = Metric.by_name(metric_key)
    if metric
      measures.find do |measure|
        measure.metric_id == metric.id && measure.rule_id.nil? && measure.rules_category_id.nil?
      end
    else
      nil
    end
  end  

  def get_url(snapshot)
    url=nil
    if @link_url and (snapshot.project.project? or snapshot.project.package?)
      url=@link_url.gsub(/_id_/, snapshot.project.key)
      unless snapshot.last?
        url = "#{url}?sid=#{snapshot.id}"
      end
    end
    url
  end

  def get_selected_metrics
    [@size_name, @color_name]
  end

  def get_html_tooltip(project, measures)
    html = "<table>"
    get_selected_metrics.each do |metric_name|
      measures.each do |measure|
        if(measure.metric.name==metric_name and measure.rules_category_id.nil?)
          metric = Metric.by_name(metric_name)
          html += "<tr><td align=left>#{escape_javascript(metric.short_name)}</td><td align=right><b>#{escape_javascript(measure.formatted_value)}</b></td></tr>"
        end
      end
    end
    html += "</table>"
    html
  end
  
  def get_color(value)
    interval = (@max - @min)/2
    mean = (@min + @max) / 2.0
    if (value > mean)
      value_percent = ((value - mean) / interval) * 100.0
      color = @max_color.mix_with(@mean_color, value_percent) if @direction >= 0
      color = @min_color.mix_with(@mean_color, value_percent) if @direction < 0
    else
      value_percent = ((mean - value) / interval) * 100.0
      color = @min_color.mix_with(@mean_color, value_percent) if @direction >= 0
      color = @max_color.mix_with(@mean_color, value_percent) if @direction < 0
    end
    color
  end

  def get_hex_color(value)
    get_color(value).html
  end

  def self.get_metrics_per_domain(metrics)
    metrics_per_domain = {}
    metrics.each do |metric|
      if metric.domain
        metrics_per_domain[metric.domain] ||= []
        metrics_per_domain[metric.domain] << metric
      end
    end

    domains = []
    metrics_per_domain.each do |dom, dom_metrics|
      domain = Domain.new
      domain.name = dom
      domain.metrics = dom_metrics.sort {|m1, m2| m1.short_name <=> m2.short_name}
      domains << domain
    end
    domains.sort {|d1, d2| d1.name <=> d2.name}
  end

end
  
class Domain
  attr_accessor :name, :metrics
end

class Sonar::HtmlOutput < Treemap::HtmlOutput
  attr_accessor(:link_url)
  
  def draw_node(node)
    return "" if node.bounds.nil?

    html = ""
    html += "<div id=\"node-#{node.id}\" style=\""
    html += "overflow: hidden; position:absolute;"
    html += "left: #{node.bounds.x1}px; top: #{node.bounds.y1}px;"
    html += "width: #{node.bounds.width}px; height: #{node.bounds.height}px;"
    html += "background-color: #FFF;"
    html += "\" class=\"node\">"
    html += "<div style='margin: 2px;background-color: #{node.color}; height: #{node.bounds.height-4}px; border: 1px solid #{node.color};' "
    if node.url and @details_at_depth==node.depth
      html += "onClick=\"document.location='#{node.url}';\" onmouseover=\"this.style.borderColor='#111';\" onmouseout=\"this.style.borderColor='#{node.color}';\""
    end
    html += ' >'
    html += draw_node_body(node)

    if(!node.children.nil? and node.children.size > 0)
      node.children.each do |c|
        html += draw_node(c)
      end
    end
    html += "</div></div>"
  end
   
  def draw_tooltips(node)
    html='<script type="text/javascript">'
    
    node.children.each do |c|
      if @details_at_depth==c.depth
        html += "new Tip($('node-#{c.id.to_s}'), '#{c.tooltip}', {title: '#{c.label}'});"
      end
    end
    html += '</script>'
    html
  end
end

